
.user h2 {
    color: #1DA1F2;
}

.user h2 .logo {
    width: 2em;
    vertical-align: middle;
}

.user .meta {
    padding: 1em;
}

.user .description {
    color: #1DA1F2;
    margin-bottom: 1em;
    font-size: 1.05rem;
}

.retweets, .favs {
    border-radius: 0.25em;
    padding-left: 0.25em;
    padding-right: 0.25em;
    background-color: rgba(0, 0, 0, 0.75);
    color:whitesmoke;
}

.quote, .status, .reply, .retweet {
    border-left: 1em solid rgba(0, 0, 0, 0.25);
}

.quote {
    background: #b3ffcc;
}

.status {
    background: #1DA1F233;
}

.reply {
    background: #ffffcc;
}

.retweet {
    background: #00000005;
}

.marker, .frequency {
    line-height: 2em;
    background: #1DA1F233;
    color: #1DA1F2;
    padding: 0.25em;
}

.frequency {
    background: #1da1f2;
    color: white;
    font-style: bold;
}



blockquote {
    margin: 0px;
    margin-top: 0.5em;
    padding: 1em;
    border-left: 0.5em solid rgba(0, 0, 0, 0.25);
    background: rgba(0, 0, 0, 0.1);
}

table thead th {
    font-size: 1rem;
    text-align: left;
    background: #1da1f2;
    color: white;
}

#footer {
    margin-top: 2em;
    text-align: center;    
    padding: 1em;
    font-size: 0.9rem;
    color: darkgrey;
}

.mdl-card {
    width: 100%;
}

.mdl-chip {
    vertical-align: middle;
}

table.user tr, table.user td {
    padding: 0.0em;
    vertical-align: top;
}

table.user tr td:first-child {
    font-weight: bold;
    padding-right: 1em;
}

table tr, table td {
    padding: 0.75em;
}

#retweet-network {
    height: 750px;
}

.inspector {
    max-height: 750px;
    overflow-y: scroll;
    border-left: 0.1em solid lightgray;
}