Found {{ users | length }} users for searchstring '{{ search }}'.
<table>
    <tbody>
        {% for u in users %}
        <tr>
            <td>{{ ("@" + u['screen_name']) | chip | link("/user/" + u['id'], classes=["filtered"]) | safe }}</td>
            <td>{{ u['name'] | safe }}</td>
            <td>{{ u['description'] | safe }}</td>
        </tr>
        {% endfor %}
    </tbody>
</table>    
    